@@ -2,13 +2,9 @@
 #
 #  Net::Server::HTTP - Extensible Perl HTTP base server
 #
-#  $Id: HTTP.pm,v 1.26 2012/06/19 16:25:33 rhandom Exp $
+#  Copyright (C) 2010-2017
 #
-#  Copyright (C) 2010-2012
-#
-#    Paul Seamons
-#    paul@seamons.com
-#    http://seamons.com/
+#    Paul Seamons <paul@seamons.com>
 #
 #  This package may be distributed under the terms of either the
 #  GNU General Public License
@@ -26,6 +22,8 @@ use IO::Handle ();
 use re 'taint'; # most of our regular expressions setting ENV should not be clearing taint
 use POSIX ();
 use Time::HiRes qw(time);
+my $has_xs_parser;
+BEGIN {$has_xs_parser = $ENV{'USE_XS_PARSER'} && eval { require HTTP::Parser::XS } };
 
 sub net_server_type { __PACKAGE__ }
 
@@ -45,7 +43,7 @@ sub max_header_size { shift->{'server'}->{'max_header_size'} }
 
 sub default_port { 80 }
 
-sub default_server_type { 'Fork' }
+sub default_server_type { 'PreFork' }
 
 sub post_configure {
     my $self = shift;
@@ -136,7 +134,7 @@ sub _tie_client_stdout {
         while ($headers->{'unparsed'} =~ s/^(.*?)\015?\012//) {
             my $line = $1;
 
-            if (!$headers && $line =~ m{^HTTP/(1.[01]) \s+ (\d+) (?: | \s+ .+)$ }x) {
+            if (!$headers->{'parsed'} && $line =~ m{^HTTP/(1.[01]) \s+ (\d+) (?: | \s+ .+)$ }x) {
                 $headers->{'status'} = [];
                 $headers->{'parsed'} .= "$line\015\012";
                 $prop->{'request_info'}->{'http_version'} = $1;
@@ -148,11 +146,11 @@ sub _tie_client_stdout {
                 $copy->send_status(@$s) if @$s;
                 $client->print($headers->{'parsed'}."\015\012");
                 $request_info->{'headers_sent'} = 1;
-                $request_info->{'response_header_size'} += length($headers->{'parsed'}+2);
+                $request_info->{'response_header_size'} += length($headers->{'parsed'})+2;
                 $request_info->{'response_size'} = length($headers->{'unparsed'});
                 return $client->print($headers->{'unparsed'});
             } elsif ($line !~ s/^(\w+(?:-(?:\w+))*):\s*//) {
-                my $invalid = ($line =~ /(.{0,40})/) ? "$1..." : '';
+                my $invalid = ($line =~ /(.{0,120})/) ? "$1..." : '';
                 $invalid =~ s/</&lt;/g;
                 die "Premature end of script headers: $invalid<br>\n";
             } else {
@@ -290,12 +288,14 @@ sub process_request {
     my $client = shift || $self->{'server'}->{'client'};
 
     my $ok = eval {
-        local $SIG{'ALRM'} = sub { die "Server Timeout\n" };
+        local $SIG{'ALRM'} = sub { die "Server Timeout on headers\n" };
         alarm($self->timeout_header);
         $self->process_headers($client);
 
+        $SIG{'ALRM'} = sub { die "Server Timeout on process\n" };
         alarm($self->timeout_idle);
         $self->process_http_request($client);
+
         alarm(0);
         1;
     };
@@ -321,41 +321,51 @@ sub process_headers {
     $ENV{'HTTPS'} = 'on' if $self->{'server'}->{'client'}->NS_proto =~ /SSL/;
 
     my ($ok, $headers) = $client->read_until($self->max_header_size, qr{\n\r?\n});
+    my ($req, $len, @parsed);
     die "Could not parse http headers successfully\n" if $ok != 1;
+    if ($has_xs_parser) {
+        $len = HTTP::Parser::XS::parse_http_request($headers, \%ENV);
+        die "Corrupt request" if $len == -1;
+        die "Incomplete request" if $len == -2;
+        $req = "$ENV{'REQUEST_METHOD'} $ENV{'REQUEST_URI'} $ENV{'SERVER_PROTOCOL'}";
+    } else {
+        ($req, my @lines) = split /\r?\n/, $headers;
+        die "Missing request\n" if ! defined $req;
 
-    my ($req, @lines) = split /\r?\n/, $headers;
+        if (!defined($req) || $req !~ m{ ^\s*(GET|POST|PUT|DELETE|PUSH|HEAD|OPTIONS)\s+(.+)\s+(HTTP/1\.[01])\s*$ }ix) {
+            die "Invalid request\n";
+        }
+        $ENV{'REQUEST_METHOD'}  = uc $1;
+        $ENV{'REQUEST_URI'}     = $2;
+        $ENV{'SERVER_PROTOCOL'} = $3;
+        $ENV{'QUERY_STRING'}    = $1 if $ENV{'REQUEST_URI'} =~ m{ \?(.*)$ }x;
+        $ENV{'PATH_INFO'}       = $1 if $ENV{'REQUEST_URI'} =~ m{^([^\?]+)};
 
-    if ($req !~ m{ ^\s*(GET|POST|PUT|DELETE|PUSH|HEAD|OPTIONS)\s+(.+)\s+HTTP/1\.[01]\s*$ }ix) {
-        die "Invalid request\n";
+        foreach my $l (@lines) {
+            my ($key, $val) = split /\s*:\s*/, $l, 2;
+            push @parsed, [$key, $val];
+            $key = uc($key);
+            $key = 'COOKIE' if $key eq 'COOKIES';
+            $key =~ y/-/_/;
+            $key =~ s/^\s+//;
+            $key = "HTTP_$key" if $key !~ /^CONTENT_(?:LENGTH|TYPE)$/;
+            $val =~ s/\s+$//;
+            if (exists $ENV{$key}) {
+                $ENV{$key} .= ", $val";
+            } else {
+                $ENV{$key} = $val;
+            }
+        }
+        $len = length $headers;
     }
-    $ENV{'REQUEST_METHOD'} = uc $1;
-    $ENV{'REQUEST_URI'}    = $2;
-    $ENV{'QUERY_STRING'}   = $1 if $ENV{'REQUEST_URI'} =~ m{ \?(.*)$ }x;
-    $ENV{'PATH_INFO'}      = $1 if $ENV{'REQUEST_URI'} =~ m{^([^\?]+)};
-    $ENV{'SCRIPT_NAME'}    = $self->script_name($ENV{'PATH_INFO'}) || '';
+    $ENV{'SCRIPT_NAME'} = $self->script_name($ENV{'PATH_INFO'}) || '';
+
     my $type = $Net::Server::HTTP::ISA[0];
     $type = $Net::Server::MultiType::ISA[0] if $type eq 'Net::Server::MultiType';
     $ENV{'NET_SERVER_TYPE'} = $type;
     $ENV{'NET_SERVER_SOFTWARE'} = $self->server_revision;
 
-    my @parsed;
-    foreach my $l (@lines) {
-        my ($key, $val) = split /\s*:\s*/, $l, 2;
-        push @parsed, [$key, $val];
-        $key = uc($key);
-        $key = 'COOKIE' if $key eq 'COOKIES';
-        $key =~ y/-/_/;
-        $key =~ s/^\s+//;
-        $key = "HTTP_$key" if $key !~ /^CONTENT_(?:LENGTH|TYPE)$/;
-        $val =~ s/\s+$//;
-        if (exists $ENV{$key}) {
-            $ENV{$key} .= ", $val";
-        } else {
-            $ENV{$key} = $val;
-        }
-    }
-
-    $self->_init_http_request_info($req, \@parsed, length($headers));
+    $self->_init_http_request_info($req, \@parsed, $len);
 }
 
 sub http_request_info { shift->{'request_info'} }
@@ -388,11 +398,11 @@ sub http_dispatch {
     my ($self, $dispatch_qr, $dispatch_table) = @_;
 
     $ENV{'PATH_INFO'} =~ s{^($dispatch_qr)(?=/|$|(?<=/))}{} or die "Dispatch not found\n";
+    $ENV{'SCRIPT_NAME'} = $1;
     if ($ENV{'PATH_INFO'}) {
         $ENV{'PATH_INFO'} = "/$ENV{'PATH_INFO'}" if $ENV{'PATH_INFO'} !~ m{^/};
         $ENV{'PATH_INFO'} =~ s/%([a-fA-F0-9]{2})/chr(hex $1)/eg;
     }
-    $ENV{'SCRIPT_NAME'} = $1;
     my $code = $self->{'dispatch'}->{$1};
     return $self->$code() if ref $code;
     $self->exec_cgi($code);
@@ -424,6 +434,7 @@ sub http_echo {
 sub post_process_request {
     my $self = shift;
     my $info = $self->{'request_info'};
+    $info->{'begin'} = time unless defined $info->{'begin'};
     $info->{'elapsed'} = time - $info->{'begin'};
     $self->SUPER::post_process_request(@_);
     $self->log_http_request($info);
@@ -537,7 +548,7 @@ sub exec_trusted_perl {
     local $!;
     my $pid = fork;
     die "Could not spawn child process: $!\n" if ! defined $pid;
-    $self->exec_fork_hook($pid, $file);
+    $self->exec_fork_hook($pid, $file, 1);
     if (!$pid) {
         if (!eval { require $file }) {
             my $err = "$@" || "Error while running trusted perl script\n";
@@ -947,7 +958,7 @@ request is closed.
 
 Defaults to undef.  If true, this represents the location of where
 the access log should be written to.  If a special value of STDERR
-is passed, the access log entry will be writting to the same location
+is passed, the access log entry will be writing to the same location
 as the ERROR log.
 
 =item access_log_format
